Type: Package
Package: rappdirs
Title: Application Directories: Determine Where to Save Data, Caches,
        and Logs
Version: 0.3.4
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("trl", "cre", "cph")),
    person("Sridhar", "Ratnakumar", role = "aut"),
    person("Trent", "Mick", role = "aut"),
    person("ActiveState", role = "cph",
           comment = "R/appdir.r, R/cache.r, R/data.r, R/log.r translated from appdirs"),
    person("Eddy", "Petrisor", role = "ctb"),
    person("Trevor", "Davis", role = c("trl", "aut"),
           comment = c(ORCID = "0000-0001-6341-4639")),
    person("Gabor", "Csardi", role = "ctb"),
    person("Gregory", "Jefferis", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: An easy way to determine which directories on the users
    computer you should use to save data, caches and logs. A port of
    Python's 'Appdirs' (<https://github.com/ActiveState/appdirs>) to R.
License: MIT + file LICENSE
URL: https://rappdirs.r-lib.org, https://github.com/r-lib/rappdirs
BugReports: https://github.com/r-lib/rappdirs/issues
Depends: R (>= 4.1)
Suggests: covr, roxygen2, testthat (>= 3.2.0), withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-05-05
Copyright: Original python appdirs module copyright (c) 2010
        ActiveState Software Inc. R port copyright Hadley Wickham,
        Posit, PBC.  See file LICENSE for details.
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-01-16 22:03:30 UTC; hadleywickham
Author: Hadley Wickham [trl, cre, cph],
  Sridhar Ratnakumar [aut],
  Trent Mick [aut],
  ActiveState [cph] (R/appdir.r, R/cache.r, R/data.r, R/log.r translated
    from appdirs),
  Eddy Petrisor [ctb],
  Trevor Davis [trl, aut] (ORCID:
    <https://orcid.org/0000-0001-6341-4639>),
  Gabor Csardi [ctb],
  Gregory Jefferis [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-01-17 08:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 08:41:51 UTC; unix
Archs: rappdirs.so.dSYM

# Generated by roxygen2: do not edit by hand

export(app_dir)
export(site_config_dir)
export(site_data_dir)
export(user_cache_dir)
export(user_config_dir)
export(user_data_dir)
export(user_log_dir)
useDynLib(rappdirs, .registration=TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# rappdirs

<!-- badges: start -->

[![R-CMD-check](https://github.com/r-lib/rappdirs/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/rappdirs/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/r-lib/rappdirs/graph/badge.svg)](https://app.codecov.io/gh/r-lib/rappdirs)
<!-- badges: end -->

`rappdirs` is a port of
[appdirs](https://github.com/ActiveState/appdirs) to R. It lets you find
the appropriate directory to save caches, logs, and data, on Linux, Mac,
and Windows. It allows you to store files that need to shared across R
sessions in a way that aligns with the [CRAN
policies](https://cran.r-project.org/web/packages/policies.html).

## Motivation

What directory should your app use for storing user data? If running on
Mac OS X, you should use:

    ~/Library/Application Support/<AppName>

If on Windows (at least English Win XP) that should be:

    C:\Documents and Settings\<User>\Application Data\Local Settings\<AppAuthor>\<AppName>

or possibly:

    C:\Documents and Settings\<User>\Application Data\<AppAuthor>\<AppName>

for [roaming
profiles](https://learn.microsoft.com/en-us/previous-versions/windows/it-pro/windows-vista/cc766489(v=ws.10))
but that is another story.

On Linux (and other Unices) the dir, according to the [XDG
spec](https://specifications.freedesktop.org/basedir/basedir-spec-latest.html)
(and subject to some interpretation), is either:

    ~/.config/<AppName>     

or possibly:

    ~/.local/share/<AppName>

## Installation

Stable version:

``` r
install.packages("rappdirs")
```

Development version:

``` r
pak::pak("r-lib/rappdirs")
```

## Usage

This kind of thing is what rappdirs is for. rappdirs will help you
choose an appropriate:

- user data dir (`user_data_dir()`)
- user config dir (`user_config_dir()`)
- user cache dir (`user_cache_dir()`)
- site data dir (`site_data_dir()`)
- user log dir (`user_log_dir()`)

For example, on Mac:

``` r
library(rappdirs)
appname <- "SuperApp"
appauthor <- "Acme"
user_config_dir(appname, appauthor)
#> [1] "~/Library/Application Support/SuperApp"
user_data_dir(appname, appauthor)
#> [1] "~/Library/Application Support/SuperApp"
site_data_dir(appname, appauthor)
#> [1] "/Library/Application Support/SuperApp"
user_cache_dir(appname, appauthor)
#> [1] "~/Library/Caches/SuperApp"
user_log_dir(appname, appauthor)
#> [1] "~/Library/Logs/SuperApp"
```

library(testthat)
library(rappdirs)

# ensure CRAN settings don't affect tests
Sys.unsetenv("R_USER_DATA_DIR")
Sys.unsetenv("R_USER_CONFIG_DIR")
Sys.unsetenv("R_USER_CACHE_DIR")

test_check("rappdirs")

#' Path to user cache directory
#'
#' @description
#' This functions uses `R_USER_CACHE_DIR` if set. Otherwise, they follow
#' platform conventions. Typical user cache directories are:
#'
#' * Mac OS X: `~/Library/Caches/<AppName>`
#' * Linux: `~/.cache/<AppName>`
#' * Win XP: `C:\\Documents and Settings\\<username>\\Local Settings\\Application Data\\<AppAuthor>\\<AppName>\\Cache`
#' * Vista: `C:\\Users\\<username>\\AppData\\Local\\<AppAuthor>\\<AppName>\\Cache`
#'
#' @section Opinion:
#' On Windows the only suggestion in the MSDN docs is that local settings go
#' in the `CSIDL_LOCAL_APPDATA` directory. This is identical to the
#' non-roaming app data dir (i.e. [user_data_dir()]). But apps typically put
#' cache data somewhere *under* this directory so `user_cache_dir()` appends
#' `Cache` to the `CSIDL_LOCAL_APPDATA` value, unless `opinion = FALSE`.
#'
#' @inheritParams user_data_dir
#' @param opinion (logical) Use `FALSE` to disable the appending of
#'   `Cache` on Windows. See discussion below.
#' @seealso [tempdir()] for a non-persistent temporary directory.
#' @export
#' @examples
#' user_cache_dir("rappdirs")
user_cache_dir <- function(appname = NULL, appauthor = appname, version = NULL,
                           opinion = TRUE, expand = TRUE, os = NULL) {
  version <- check_version(version, appname, expand)

  base <- base_path(os, "CACHE",
    win  = win_path("local"),
    mac  = "~/Library/Caches",
    unix = Sys.getenv("XDG_CACHE_HOME", "~/.cache")
  )

  switch(check_os(os),
    win = file_path(base, appauthor, appname, version, if (opinion) "Cache"),
    mac = file_path(base, appname, version),
    unix = file_path(base, appname, version)
  )
}

test_that("works on mac and linux", {
  withr::local_envvar(XDG_CACHE_HOME = NA)
  expect_equal(user_cache_dir("R", os = "unix"), "~/.cache/R")
  expect_equal(user_cache_dir("R", os = "mac"), "~/Library/Caches/R")
})

test_that("can override with R_USER_CACHE_DIR", {
  withr::local_envvar(R_USER_CACHE_DIR = "/test")
  expect_equal(user_cache_dir("R", os = "mac"), "/test/R")
})
test_that("works on windows simulation", {
  skip_on_os("windows")
  expect_equal(
    user_cache_dir("R", os = "win"),
    "<USERPROFILE>/Local Settings/Application Data/R/R/Cache"
  )
})

#' Path to user log directory
#'
#' Typical user cache directories are:
#'
#' \itemize{
#'   \item Mac OS X: \file{~/Library/Logs/<AppName>}
#'   \item Unix: \file{~/.cache/<AppName>/log}, or under
#'     \\env{$XDG_CACHE_HOME} if defined
#'   \item Win XP:  \file{C:\\Documents and Settings\\<username>\\Local Settings\\Application Data\\<AppAuthor>\\<AppName>\\Logs}
#'   \item Vista:
#'     \file{C:\\Users\\<username>\\AppData\\Local\\<AppAuthor>\\<AppName>\\Logs}
#' }
#'
#' On Windows the only suggestion in the MSDN docs is that local settings
#' go in the `CSIDL_LOCAL_APPDATA` directory.
#'
#' @section Opinion:
#' This function appends \file{Logs} to the `CSIDL_LOCAL_APPDATA`
#' value for Windows and appends \file{log} to the user cache dir for Unix.
#' This can be disabled with the `opinion = FALSE` option.
#'
#' @inheritParams user_data_dir
#' @param opinion (logical) can be `FALSE` to disable the appending of
#'   \file{Logs} to the base app data dir for Windows, and \file{log} to the
#'   base cache dir for Unix. See discussion below.
#' @examples
#' user_log_dir()
#' @export
user_log_dir <- function(appname = NULL, appauthor = appname, version = NULL,
                         opinion = TRUE, expand = TRUE, os = NULL) {
  version <- check_version(version, appname, expand)

  switch(check_os(os),
    win = file_path(
      win_path("local"), appauthor, appname, version,
      if (opinion) "Logs"
    ),
    mac = file_path("~/Library/Logs", appname, version),
    unix = file_path(
      Sys.getenv("XDG_CACHE_HOME", "~/.cache"),
      appname, version, if (opinion) "log"
    )
  )
}

test_that("user_cache_dir works as expected", {
  expect_equal(user_log_dir("R", os = "mac"), "~/Library/Logs/R")

  withr::local_envvar(XDG_CACHE_HOME = "/cache")
  expect_equal(user_log_dir("R", os = "unix"), "/cache/R/log")
  expect_equal(user_log_dir("R", os = "unix", opinion = FALSE), "/cache/R")
})

test_that("user_cache_dir works with windows simulation", {
  skip_on_os("windows")
  expect_equal(
    user_log_dir("R", os = "win"),
    "<USERPROFILE>/Local Settings/Application Data/R/R/Logs"
  )
})

check_os <- function(os) {
  if (is.null(os)) {
    get_os()
  } else {
    if (length(os) != 1 || !is.character(os)) {
      stop("`os` must be a string", call. = FALSE)
    }
    if (!os %in% c("win", "mac", "unix")) {
      stop("`os` must be one of 'win', 'mac', 'unix'", call. = FALSE)
    }
    os
  }
}

get_os <- function() {
  if (.Platform$OS.type == "windows") {
    "win"
  } else if (Sys.info()["sysname"] == "Darwin") {
    "mac"
  } else {
    "unix"
  }
}

file_path <- function(...) {
  paste(c(...), collapse = .Platform$file.sep)
}

"%||%" <- function(a, b) if (is.null(a)) b else a

base_path <- function(os, type, win, mac, unix) {
  name <- paste0("R_USER_", type, "_DIR")
  val <- Sys.getenv(name)

  if (!identical(val, "")) {
    val
  } else {
    switch(check_os(os), win = win, mac = mac, unix = unix)
  }
}

win_path <- function(type_appdata = "common") {
  CSIDL_APPDATA <- 26L
  CSIDL_COMMON_APPDATA <- 35L
  CSIDL_LOCAL_APPDATA <- 28L

  switch(type_appdata,
    roaming = win_path_csidl(CSIDL_APPDATA) %||% win_path_env("roaming"),
    local = win_path_csidl(CSIDL_LOCAL_APPDATA) %||% win_path_env("local"),
    common = win_path_csidl(CSIDL_COMMON_APPDATA) %||% win_path_env("common")
  )
}

#' @useDynLib rappdirs, .registration=TRUE
win_path_csidl <- function(csidl) {
  stopifnot(is.integer(csidl), length(csidl) == 1)
  path <- .Call(win_path_, csidl, PACKAGE = "rappdirs")
  path
}

# How to get reasonable window paths via environmental variables
win_path_env <- function(type) {
  if (type == "roaming") {
    env_fallback("APPDATA")
  } else if (type == "local") {
    path <- Sys.getenv("LOCALAPPDATA", unset = NA)
    if (is.na(path)) { # environmental variable not defined in XP
      path <- file.path(
        env_fallback("USERPROFILE"),
        "Local Settings", "Application Data"
      )
    }
    path
  } else if (type == "common") {
    path <- Sys.getenv("PROGRAMDATA", unset = NA)
    if (is.na(path)) {
      path <- file.path(env_fallback("ALLUSERPROFILE"), "Application Data")
    }
    path
  } else {
    stop("invalid `type` argument")
  }
}

env_fallback <- function(env) {
  val <- Sys.getenv(env)

  if (identical(val, "")) {
    if (get_os() == "win") {
      stop("Can't find envvar '", env, "'", call. = FALSE)
    } else {
      # Fall back so examples still work when not on windows
      paste0("<", env, ">")
    }
  } else {
    val
  }
}

# version -----------------------------------------------------------------

check_version <- function(version, appname, expand = FALSE) {
  if (is.null(appname) && !is.null(version)) {
    warning("version is ignored when appname is null", call. = FALSE)
    NULL
  } else {
    if (expand) {
      version <- expand_r_libs_specifiers(version)
    }
    version
  }
}

expand_r_libs_specifiers <- function(x) {
  if (is.null(x)) {
    return(NULL)
  }
  rversion <- getRversion()
  x <- gsub_special("%V", rversion, x)
  x <- gsub_special("%v", paste(rversion$major, rversion$minor, sep = "."), x)
  x <- gsub_special("%p", R.version$platform, x)
  x <- gsub_special("%o", R.version$os, x)
  x <- gsub_special("%a", R.version$arch, x)
  x <- gsub("%%", "%", x)
  x
}

gsub_special <- function(pattern, replacement, x) {
  gsub(paste0("([^%]|^)", pattern), paste0("\\1", replacement), x)
}

test_that("expand_r_libs_specifiers works as expected", {
  expect_equal(expand_r_libs_specifiers("%V"), as.character(getRversion()))
  expect_equal(expand_r_libs_specifiers("%%V"), "%V")
  expect_equal(expand_r_libs_specifiers(NULL), NULL)
})

test_that("file_path drops NULLs", {
  expect_equal(file_path("a", NULL, "b"), "a/b")
})

test_that("file_path is not vectorised", {
  expect_equal(file_path(c("a", "b"), c("c", "d")), "a/b/c/d")
})

test_that("windows fallbacks work", {
  skip_on_os("windows")
  withr::local_envvar(LOCALAPPDATA = NA, PROGRAMDATA = NA)

  expect_equal(win_path_env("roaming"), "<APPDATA>")

  expect_equal(win_path_env("local"), "<USERPROFILE>/Local Settings/Application Data")
  withr::local_envvar(LOCALAPPDATA = "<LOCALAPPDATA>")
  expect_equal(win_path_env("local"), "<LOCALAPPDATA>")

  expect_equal(win_path_env("common"), "<ALLUSERPROFILE>/Application Data")
  withr::local_envvar(PROGRAMDATA = "<PROGRAMDATA>")
  expect_equal(win_path_env("common"), "<PROGRAMDATA>")

  expect_error(win_path_env("foo"), "invalid")
})

test_that("check_version works as expected", {
  expect_snapshot({
    expect_equal(check_version("1", NULL), NULL)
    expect_equal(check_version("1", "R"), "1")
  })
})

#' Record app information in a convenient object
#'
#' @description
#' Has methods:
#' * `$cache()`
#' * `$log()`
#' * `$data()`
#' * `$config()`
#' * `$site_data()`
#' * `$site_config()`
#'
#' @inheritParams user_data_dir
#' @keywords internal
#' @export
#' @examples
#' ggplot2_app <- app_dir("ggplot2", "hadley")
#' ggplot2_app$cache()
#' ggplot2_app$log()
#' ggplot2_app$data()
#' ggplot2_app$config()
#' ggplot2_app$site_config()
#' ggplot2_app$site_data()
app_dir <- function(appname = NULL, appauthor = appname, version = NULL, expand = TRUE, os = NULL) {
  os <- check_os(os)
  list(
    cache = function(opinion = TRUE) user_cache_dir(appname, appauthor, version, opinion, expand, os),
    log = function(opinion = TRUE) user_log_dir(appname, appauthor, version, opinion, expand, os),
    data = function(roaming = FALSE) user_data_dir(appname, appauthor, version, roaming, expand, os),
    config = function(roaming = FALSE) user_config_dir(appname, appauthor, version, roaming, expand, os),
    site_data = function(multipath = FALSE) site_data_dir(appname, appauthor, version, multipath, expand, os),
    site_config = function(multipath = FALSE) site_config_dir(appname, appauthor, version, multipath, expand, os),
    os = os
  )
}

test_that("appdir works as expected", {
  expect_snapshot({
    app <- app_dir("ggplot2", "hadley", os = "mac")
    app$cache()
    app$log()
    app$data()
    app$config()
    app$site_data()
    app$site_config()
  })
})

#' Path to user config/data directories
#'
#' @description
#' `user_data_dir()` returns path to the user-specific data directory and
#' `user_config_dir()` returns full path to the user-specific configuration
#' directory. These are the same on Windows and Mac but different on Linux.
#'
#' These functions first use `R_USER_DATA_DIR` and `R_USER_CONFIG_DIR` if set.
#' Otherwise, they follow platform conventions. Typical user config and data
#' directories are:
#'
#' * Mac OS X: `~/Library/Application Support/<AppName>`
#' * Win XP (not roaming): `C:\\Documents and Settings\\<username>\\Data\\<AppAuthor>\\<AppName>`
#' * Win XP (roaming): `C:\\Documents and Settings\\<username>\\Local Settings\\Data\\<AppAuthor>\\<AppName>`
#' * Win 7 (not roaming): `C:\\Users\\<username>\\AppData\\Local\\<AppAuthor>\\<AppName>`
#' * Win 7 (roaming): `C:\\Users\\<username>\\AppData\\Roaming\\<AppAuthor>\\<AppName>`
#'
#' Only Linux makes the distinction between config and data:
#'
#' * Data: `~/.local/share/<AppName>`
#' * Config: `~/.config/<AppName>`
#'
#' @param appname is the name of application. If NULL, just the system
#'     directory is returned.
#' @param appauthor (only required and used on Windows) is the name of the
#'     appauthor or distributing body for this application. Typically
#'     it is the owning company name. This falls back to appname.
#' @param version is an optional version path element to append to the
#'     path. You might want to use this if you want multiple versions
#'     of your app to be able to run independently. If used, this
#'     would typically be `"<major>.<minor>"`. Only applied when appname
#'     is not NULL.
#' @param roaming (logical, default `FALSE`) can be set `TRUE` to
#'     use the Windows roaming appdata directory. That means that for users on
#'     a Windows network setup for roaming profiles, this user data will be
#'     sync'd on login. See
#'     <https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-vista/cc766489(v=ws.10)>
#'     for a discussion of issues.
#' @param os Operating system whose conventions are used to construct the
#'     requested directory. Possible values are "win", "mac", "unix". If `NULL`
#'     (the default) then the current OS will be used.
#' @param expand If TRUE (the default) will expand the `R_LIBS` specifiers with their equivalents.
#'      See [R_LIBS()] for list of all possibly specifiers.
#' @export
#' @examples
#' user_data_dir("rappdirs")
#'
#' user_config_dir("rappdirs", roaming = TRUE, os = "win")
#' user_config_dir("rappdirs", roaming = FALSE, os = "win")
#' user_config_dir("rappdirs", os = "unix")
#' user_config_dir("rappdirs", os = "mac")
#' user_config_dir("rappdirs", version = "%p-platform/%v")
user_data_dir <- function(appname = NULL, appauthor = appname, version = NULL,
                          roaming = FALSE, expand = TRUE, os = NULL) {
  version <- check_version(version, appname, expand)

  base <- base_path(os, "DATA",
    win  = win_path(ifelse(roaming, "roaming", "local")),
    mac  = "~/Library/Application Support",
    unix = Sys.getenv("XDG_DATA_HOME", "~/.local/share")
  )
  switch(check_os(os),
    win = file_path(base, appauthor, appname, version),
    mac = file_path(base, appname, version),
    unix = file_path(base, appname, version)
  )
}

#' @rdname user_data_dir
#' @export
user_config_dir <- function(appname = NULL, appauthor = appname, version = NULL,
                            roaming = TRUE, expand = TRUE, os = NULL) {
  version <- check_version(version, appname, expand)

  base <- base_path(os, "CONFIG",
    win  = win_path(ifelse(roaming, "roaming", "local")),
    mac  = "~/Library/Application Support",
    unix = Sys.getenv("XDG_CONFIG_HOME", "~/.config")
  )

  switch(check_os(os),
    win = file_path(base, appauthor, appname, version),
    mac = file_path(base, appname, version),
    unix = file_path(base, appname, version)
  )
}


#' Path to shared data/config directories
#'
#' `site_data_dir` returns full path to the user-shared data dir for this application.
#' `site_config_dir` returns full path to the user-specific configuration directory for this application
#' which returns the same path as site data directory in Windows and Mac but a different one for Unix.
#' Typical user-shared data directories are:
#'
#' * Mac OS X:  `/Library/Application Support/<AppName>`
#' * Unix:      `/usr/local/share:/usr/share/`
#' * Win XP:    `C:\\Documents and Settings\\All Users\\Application Data\\<AppAuthor>\\<AppName>`
#' * Vista:     (Fail! `C:\\ProgramData` is a hidden *system* directory on Vista.)
#' * Win 7:     `C:\\ProgramData\\<AppAuthor>\\<AppName>`. Hidden, but writeable on Win 7.
#'
#' Unix also specifies a separate location for user-shared configuration data in \env{$XDG_CONFIG_DIRS}.
#'
#' * Unix: `/etc/xdg/<AppName>`, in \env{$XDG_CONFIG_HOME} if defined
#'
#' For Unix, this returns the first default.  Set the `multipath=TRUE` to guarantee returning all directories.
#'
#' @inheritParams user_data_dir
#' @param multipath is an optional parameter only applicable to *nix
#'       which indicates that the entire list of data dirs should be returned
#'       By default, the first directory is returned
#' @section Warning:
#' Do not use this on Windows. See the note above for why.
#' @export
site_data_dir <- function(appname = NULL, appauthor = appname, version = NULL,
                          multipath = FALSE, expand = TRUE, os = NULL) {
  version <- check_version(version, appname, expand)

  switch(check_os(os),
    win = file_path(win_path("common"), appauthor, appname, version),
    mac = file_path("/Library/Application Support", appname, version),
    unix = file_path_site_unix(
      Sys.getenv("XDG_DATA_DIRS", "/usr/local/share:/usr/share"),
      appname, version, multipath
    )
  )
}

#' @rdname site_data_dir
#' @export
site_config_dir <- function(appname = NULL, appauthor = appname, version = NULL,
                            multipath = FALSE, expand = TRUE, os = NULL) {
  version <- check_version(version, appname, expand)

  switch(check_os(os),
    win = file_path(win_path("common"), appauthor, appname, version),
    mac = file_path("/Library/Application Support", appname, version),
    unix = file_path_site_unix(
      Sys.getenv("XDG_CONFIG_DIRS", "/etc/xdg"),
      appname, version, multipath
    )
  )
}

# wrapper with `multipath` and use `parse_path_string` for cleaner switch statement
file_path_site_unix <- function(sys_getenv, appname, version, multipath = FALSE) {
  paths <- parse_path_string(sys_getenv)
  if (multipath) {
    vapply(paths, file_path, appname, version,
      FUN.VALUE = character(1),
      USE.NAMES = FALSE
    )
  } else {
    file_path(paths[[1]], appname, version)
  }
}

parse_path_string <- function(path, sep = ":") {
  unique(strsplit(path, sep)[[1]])
}

test_that("default paths work on mac", {
  expect_equal(user_data_dir("R", os = "mac"), "~/Library/Application Support/R")
  expect_equal(user_config_dir("R", os = "mac"), "~/Library/Application Support/R")
  expect_equal(site_data_dir("R", os = "mac"), "/Library/Application Support/R")
  expect_equal(site_config_dir("R", os = "mac"), "/Library/Application Support/R")
})

test_that("default paths work on linux", {
  withr::local_envvar(
    XDG_DATA_HOME = NA,
    XDG_CONFIG_HOME = NA,
    XDG_DATA_DIRS = NA,
    XDG_CONFIG_DIRS = NA,
  )

  expect_equal(user_data_dir("R", os = "unix"), "~/.local/share/R")
  expect_equal(user_config_dir("R", os = "unix"), "~/.config/R")
  expect_equal(site_data_dir("R", os = "unix"), "/usr/local/share/R")
  expect_equal(site_config_dir("R", os = "unix"), "/etc/xdg/R")
})

test_that("can override linux paths with envvars", {
  withr::local_envvar(
    XDG_DATA_HOME = "A",
    XDG_CONFIG_HOME = "B",
    XDG_DATA_DIRS = "C",
    XDG_CONFIG_DIRS = "D",
  )

  expect_equal(user_data_dir("R", os = "unix"), "A/R")
  expect_equal(user_config_dir("R", os = "unix"), "B/R")
  expect_equal(site_data_dir("R", os = "unix"), "C/R")
  expect_equal(site_config_dir("R", os = "unix"), "D/R")
})

test_that("can override with R_USER_DATA_DIR", {
  withr::local_envvar(R_USER_DATA_DIR = "/test")
  expect_equal(user_data_dir("R", os = "mac"), "/test/R")

  withr::local_envvar(R_USER_CONFIG_DIR = "/test")
  expect_equal(user_config_dir("R", os = "mac"), "/test/R")
})

test_that("can optionally use all XDG_DATA_DIRS", {
  withr::local_envvar(XDG_DATA_DIRS = "/usr/local/share:/usr/share")
  expect_equal(
    site_data_dir("R", os = "unix", multipath = TRUE),
    c("/usr/local/share/R", "/usr/share/R")
  )
  expect_equal(
    site_data_dir("R", os = "unix", multipath = FALSE),
    "/usr/local/share/R"
  )
})


test_that("default paths work in windows simulation", {
  skip_on_os("windows")
  expect_equal(user_data_dir("R", os = "win"), "<USERPROFILE>/Local Settings/Application Data/R/R")
  expect_equal(user_config_dir("R", os = "win"), "<APPDATA>/R/R")
  expect_equal(site_data_dir("R", os = "win"), "<ALLUSERPROFILE>/Application Data/R/R")
  expect_equal(site_config_dir("R", os = "win"), "<ALLUSERPROFILE>/Application Data/R/R")
})

test_that("can override windows paths with env vars", {
  skip_on_os("windows")
  withr::local_envvar(LOCALAPPDATA = NA, PROGRAMDATA = NA)

  withr::local_envvar(APPDATA = "C:/config")
  expect_equal(
    user_config_dir("R", os = "win", roaming = TRUE),
    "C:/config/R/R"
  )

  withr::local_envvar("USERPROFILE" = "C:/config1")
  expect_equal(
    user_config_dir("R", os = "win", roaming = FALSE),
    "C:/config1/Local Settings/Application Data/R/R"
  )

  withr::local_envvar("LOCALAPPDATA" = "C:/config2")
  expect_equal(
    user_config_dir("R", os = "win", roaming = FALSE),
    "C:/config2/R/R"
  )
})


test_that("can expand versioned paths", {
  expect_equal(
    site_data_dir("R", version = "%V", os = "mac", expand = TRUE),
    file.path("/Library/Application Support/R", as.character(getRversion()))
  )
  expect_equal(
    site_data_dir("R", version = "%V", os = "mac", expand = FALSE),
    "/Library/Application Support/R/%V"
  )
})

# linux -------------------------------------------------------------------

# windows fallbacks work

    Code
      win_path_env("foo")
    Condition
      Error in `win_path_env()`:
      ! invalid `type` argument

# check_version works as expected

    Code
      expect_equal(check_version("1", NULL), NULL)
    Condition
      Warning:
      version is ignored when appname is null
    Code
      expect_equal(check_version("1", "R"), "1")

# appdir works as expected

    Code
      app <- app_dir("ggplot2", "hadley", os = "mac")
    Code
      app$cache()
    Output
      [1] "~/Library/Caches/ggplot2"
    Code
      app$log()
    Output
      [1] "~/Library/Logs/ggplot2"
    Code
      app$data()
    Output
      [1] "~/Library/Application Support/ggplot2"
    Code
      app$config()
    Output
      [1] "~/Library/Application Support/ggplot2"
    Code
      app$site_data()
    Output
      [1] "/Library/Application Support/ggplot2"
    Code
      app$site_config()
    Output
      [1] "/Library/Application Support/ggplot2"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\name{user_data_dir}
\alias{user_data_dir}
\alias{user_config_dir}
\title{Path to user config/data directories}
\usage{
user_data_dir(
  appname = NULL,
  appauthor = appname,
  version = NULL,
  roaming = FALSE,
  expand = TRUE,
  os = NULL
)

user_config_dir(
  appname = NULL,
  appauthor = appname,
  version = NULL,
  roaming = TRUE,
  expand = TRUE,
  os = NULL
)
}
\arguments{
\item{appname}{is the name of application. If NULL, just the system
directory is returned.}

\item{appauthor}{(only required and used on Windows) is the name of the
appauthor or distributing body for this application. Typically
it is the owning company name. This falls back to appname.}

\item{version}{is an optional version path element to append to the
path. You might want to use this if you want multiple versions
of your app to be able to run independently. If used, this
would typically be \code{"<major>.<minor>"}. Only applied when appname
is not NULL.}

\item{roaming}{(logical, default \code{FALSE}) can be set \code{TRUE} to
use the Windows roaming appdata directory. That means that for users on
a Windows network setup for roaming profiles, this user data will be
sync'd on login. See
\url{https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-vista/cc766489(v=ws.10)}
for a discussion of issues.}

\item{expand}{If TRUE (the default) will expand the \code{R_LIBS} specifiers with their equivalents.
See \code{\link[=R_LIBS]{R_LIBS()}} for list of all possibly specifiers.}

\item{os}{Operating system whose conventions are used to construct the
requested directory. Possible values are "win", "mac", "unix". If \code{NULL}
(the default) then the current OS will be used.}
}
\description{
\code{user_data_dir()} returns path to the user-specific data directory and
\code{user_config_dir()} returns full path to the user-specific configuration
directory. These are the same on Windows and Mac but different on Linux.

These functions first use \code{R_USER_DATA_DIR} and \code{R_USER_CONFIG_DIR} if set.
Otherwise, they follow platform conventions. Typical user config and data
directories are:
\itemize{
\item Mac OS X: \verb{~/Library/Application Support/<AppName>}
\item Win XP (not roaming): \verb{C:\\\\Documents and Settings\\\\<username>\\\\Data\\\\<AppAuthor>\\\\<AppName>}
\item Win XP (roaming): \verb{C:\\\\Documents and Settings\\\\<username>\\\\Local Settings\\\\Data\\\\<AppAuthor>\\\\<AppName>}
\item Win 7 (not roaming): \verb{C:\\\\Users\\\\<username>\\\\AppData\\\\Local\\\\<AppAuthor>\\\\<AppName>}
\item Win 7 (roaming): \verb{C:\\\\Users\\\\<username>\\\\AppData\\\\Roaming\\\\<AppAuthor>\\\\<AppName>}
}

Only Linux makes the distinction between config and data:
\itemize{
\item Data: \verb{~/.local/share/<AppName>}
\item Config: \verb{~/.config/<AppName>}
}
}
\examples{
user_data_dir("rappdirs")

user_config_dir("rappdirs", roaming = TRUE, os = "win")
user_config_dir("rappdirs", roaming = FALSE, os = "win")
user_config_dir("rappdirs", os = "unix")
user_config_dir("rappdirs", os = "mac")
user_config_dir("rappdirs", version = "\%p-platform/\%v")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rappdirs-package.r
\docType{package}
\name{rappdirs-package}
\alias{rappdirs}
\alias{rappdirs-package}
\title{rappdirs: Application Directories: Determine Where to Save Data,
    Caches, and Logs}
\description{
An easy way to determine which directories on the
    users computer you should use to save data, caches and logs. A port of
    Python's 'Appdirs' (<https://github.com/ActiveState/appdirs>) to
    R.
}
\section{Main functions}{

\itemize{
\item \code{\link[=user_data_dir]{user_data_dir()}}
\item \code{\link[=user_config_dir]{user_config_dir()}}
\item \code{\link[=user_cache_dir]{user_cache_dir()}}
\item \code{\link[=site_data_dir]{site_data_dir()}}
\item \code{\link[=user_log_dir]{user_log_dir()}}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://rappdirs.r-lib.org}
  \item \url{https://github.com/r-lib/rappdirs}
  \item Report bugs at \url{https://github.com/r-lib/rappdirs/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@rstudio.com} [translator, copyright holder]

Authors:
\itemize{
  \item Sridhar Ratnakumar
  \item Trent Mick
  \item Trevor Davis [translator]
}

Other contributors:
\itemize{
  \item RStudio [copyright holder]
  \item ActiveState (R/appdir.r, R/cache.r, R/data.r, R/log.r translated from appdirs) [copyright holder]
  \item Eddy Petrisor [contributor]
  \item Gabor Csardi [contributor]
  \item Gregory Jefferis [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.r
\name{site_data_dir}
\alias{site_data_dir}
\alias{site_config_dir}
\title{Path to shared data/config directories}
\usage{
site_data_dir(
  appname = NULL,
  appauthor = appname,
  version = NULL,
  multipath = FALSE,
  expand = TRUE,
  os = NULL
)

site_config_dir(
  appname = NULL,
  appauthor = appname,
  version = NULL,
  multipath = FALSE,
  expand = TRUE,
  os = NULL
)
}
\arguments{
\item{appname}{is the name of application. If NULL, just the system
directory is returned.}

\item{appauthor}{(only required and used on Windows) is the name of the
appauthor or distributing body for this application. Typically
it is the owning company name. This falls back to appname.}

\item{version}{is an optional version path element to append to the
path. You might want to use this if you want multiple versions
of your app to be able to run independently. If used, this
would typically be \code{"<major>.<minor>"}. Only applied when appname
is not NULL.}

\item{multipath}{is an optional parameter only applicable to *nix
which indicates that the entire list of data dirs should be returned
By default, the first directory is returned}

\item{expand}{If TRUE (the default) will expand the \code{R_LIBS} specifiers with their equivalents.
See \code{\link[=R_LIBS]{R_LIBS()}} for list of all possibly specifiers.}

\item{os}{Operating system whose conventions are used to construct the
requested directory. Possible values are "win", "mac", "unix". If \code{NULL}
(the default) then the current OS will be used.}
}
\description{
\code{site_data_dir} returns full path to the user-shared data dir for this application.
\code{site_config_dir} returns full path to the user-specific configuration directory for this application
which returns the same path as site data directory in Windows and Mac but a different one for Unix.
Typical user-shared data directories are:
}
\details{
\itemize{
\item Mac OS X:  \verb{/Library/Application Support/<AppName>}
\item Unix:      \verb{/usr/local/share:/usr/share/}
\item Win XP:    \verb{C:\\\\Documents and Settings\\\\All Users\\\\Application Data\\\\<AppAuthor>\\\\<AppName>}
\item Vista:     (Fail! \verb{C:\\\\ProgramData} is a hidden \emph{system} directory on Vista.)
\item Win 7:     \verb{C:\\\\ProgramData\\\\<AppAuthor>\\\\<AppName>}. Hidden, but writeable on Win 7.
}

Unix also specifies a separate location for user-shared configuration data in \env{$XDG_CONFIG_DIRS}.
\itemize{
\item Unix: \verb{/etc/xdg/<AppName>}, in \env{$XDG_CONFIG_HOME} if defined
}

For Unix, this returns the first default.  Set the \code{multipath=TRUE} to guarantee returning all directories.
}
\section{Warning}{

Do not use this on Windows. See the note above for why.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/appdir.r
\name{app_dir}
\alias{app_dir}
\title{Record app information in a convenient object}
\usage{
app_dir(
  appname = NULL,
  appauthor = appname,
  version = NULL,
  expand = TRUE,
  os = NULL
)
}
\arguments{
\item{appname}{is the name of application. If NULL, just the system
directory is returned.}

\item{appauthor}{(only required and used on Windows) is the name of the
appauthor or distributing body for this application. Typically
it is the owning company name. This falls back to appname.}

\item{version}{is an optional version path element to append to the
path. You might want to use this if you want multiple versions
of your app to be able to run independently. If used, this
would typically be \code{"<major>.<minor>"}. Only applied when appname
is not NULL.}

\item{expand}{If TRUE (the default) will expand the \code{R_LIBS} specifiers with their equivalents.
See \code{\link[=R_LIBS]{R_LIBS()}} for list of all possibly specifiers.}

\item{os}{Operating system whose conventions are used to construct the
requested directory. Possible values are "win", "mac", "unix". If \code{NULL}
(the default) then the current OS will be used.}
}
\description{
Has methods:
\itemize{
\item \verb{$cache()}
\item \verb{$log()}
\item \verb{$data()}
\item \verb{$config()}
\item \verb{$site_data()}
\item \verb{$site_config()}
}
}
\examples{
ggplot2_app <- app_dir("ggplot2", "hadley")
ggplot2_app$cache()
ggplot2_app$log()
ggplot2_app$data()
ggplot2_app$config()
ggplot2_app$site_config()
ggplot2_app$site_data()
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.r
\name{user_cache_dir}
\alias{user_cache_dir}
\title{Path to user cache directory}
\usage{
user_cache_dir(
  appname = NULL,
  appauthor = appname,
  version = NULL,
  opinion = TRUE,
  expand = TRUE,
  os = NULL
)
}
\arguments{
\item{appname}{is the name of application. If NULL, just the system
directory is returned.}

\item{appauthor}{(only required and used on Windows) is the name of the
appauthor or distributing body for this application. Typically
it is the owning company name. This falls back to appname.}

\item{version}{is an optional version path element to append to the
path. You might want to use this if you want multiple versions
of your app to be able to run independently. If used, this
would typically be \code{"<major>.<minor>"}. Only applied when appname
is not NULL.}

\item{opinion}{(logical) Use \code{FALSE} to disable the appending of
\code{Cache} on Windows. See discussion below.}

\item{expand}{If TRUE (the default) will expand the \code{R_LIBS} specifiers with their equivalents.
See \code{\link[=R_LIBS]{R_LIBS()}} for list of all possibly specifiers.}

\item{os}{Operating system whose conventions are used to construct the
requested directory. Possible values are "win", "mac", "unix". If \code{NULL}
(the default) then the current OS will be used.}
}
\description{
This functions uses \code{R_USER_CACHE_DIR} if set. Otherwise, they follow
platform conventions. Typical user cache directories are:
\itemize{
\item Mac OS X: \verb{~/Library/Caches/<AppName>}
\item Linux: \verb{~/.cache/<AppName>}
\item Win XP: \verb{C:\\\\Documents and Settings\\\\<username>\\\\Local Settings\\\\Application Data\\\\<AppAuthor>\\\\<AppName>\\\\Cache}
\item Vista: \verb{C:\\\\Users\\\\<username>\\\\AppData\\\\Local\\\\<AppAuthor>\\\\<AppName>\\\\Cache}
}
}
\section{Opinion}{

On Windows the only suggestion in the MSDN docs is that local settings go
in the \code{CSIDL_LOCAL_APPDATA} directory. This is identical to the
non-roaming app data dir (i.e. \code{\link[=user_data_dir]{user_data_dir()}}). But apps typically put
cache data somewhere \emph{under} this directory so \code{user_cache_dir()} appends
\code{Cache} to the \code{CSIDL_LOCAL_APPDATA} value, unless \code{opinion = FALSE}.
}

\examples{
user_cache_dir("rappdirs")
}
\seealso{
\code{\link[=tempdir]{tempdir()}} for a non-persistent temporary directory.
}
